/*
 * Copyright (C) 2005-2006 Atmel Corporation
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */
#ifndef __ASM_AVR32_DMA_CONTROLLER_H
#define __ASM_AVR32_DMA_CONTROLLER_H

#include <linux/device.h>

#define DMA_DIR_MEM_TO_MEM		0x0000
#define DMA_DIR_MEM_TO_PERIPH		0x0001
#define DMA_DIR_PERIPH_TO_MEM		0x0002
#define DMA_DIR_PERIPH_TO_PERIPH	0x0003

#define DMA_WIDTH_8BIT			0
#define DMA_WIDTH_16BIT			1
#define DMA_WIDTH_32BIT			2

struct dma_request {
	struct dma_controller *dmac;
	struct list_head list;

	unsigned short channel;

	void (*xfer_complete)(struct dma_request *req);
	void (*block_complete)(struct dma_request *req);
	void (*error)(struct dma_request *req);
};

struct dma_request_sg {
	struct dma_request req;

	int nr_sg;
	struct scatterlist *sg;
	unsigned long block_size;
	unsigned int nr_blocks;

	dma_addr_t data_reg;
	unsigned short periph_id;

	unsigned char direction;
	unsigned char width;
};
#define to_dma_request_sg(_req)				\
	container_of(_req, struct dma_request_sg, req)

struct dma_request_cyclic {
	struct dma_request req;

        int periods;
	unsigned long buffer_size;

        dma_addr_t buffer_start;
	dma_addr_t data_reg;

	unsigned short periph_id;
	unsigned char direction;
	unsigned char width;

        void *dev_id;
};
#define to_dma_request_cyclic(_req)				\
	container_of(_req, struct dma_request_cyclic, req)

struct dma_request_memcpy {
	struct dma_request req;

	dma_addr_t src_addr;
	unsigned int src_width;
	unsigned int src_stride;

	dma_addr_t dst_addr;
	unsigned int dst_width;
	unsigned int dst_stride;

	size_t length;

	unsigned short src_reverse:1;
	unsigned short dst_reverse:1;
};
#define to_dma_request_memcpy(_req)				\
	container_of(_req, struct dma_request_memcpy, req)

struct dma_controller {
	struct list_head list;
	int id;
	struct device *dev;

	int (*alloc_channel)(struct dma_controller *dmac);
	void (*release_channel)(struct dma_controller *dmac,
				int channel);
	int (*prepare_request_sg)(struct dma_controller *dmac,
				  struct dma_request_sg *req);
        int (*prepare_request_cyclic)(struct dma_controller *dmac,
				      struct dma_request_cyclic *req);
	int (*prepare_request_memcpy)(struct dma_controller *dmac,
				      struct dma_request_memcpy *req);
	int (*start_request)(struct dma_controller *dmac,
			     unsigned int channel);
	int (*stop_request)(struct dma_controller *dmac,
                            unsigned int channel);
        dma_addr_t (*get_current_pos)(struct dma_controller *dmac,
                                      unsigned int channel);
};

static inline int
dma_alloc_channel(struct dma_controller *dmac)
{
	return dmac->alloc_channel(dmac);
}

static inline void
dma_release_channel(struct dma_controller *dmac, int chan)
{
	dmac->release_channel(dmac, chan);
}

static inline int
dma_prepare_request_sg(struct dma_controller *dmac,
		       struct dma_request_sg *req)
{
	return dmac->prepare_request_sg(dmac, req);
}

static inline int
dma_prepare_request_cyclic(struct dma_controller *dmac,
			   struct dma_request_cyclic *req)
{
	return dmac->prepare_request_cyclic(dmac, req);
}

static inline int
dma_prepare_request_memcpy(struct dma_controller *dmac,
			   struct dma_request_memcpy *req)
{
	return dmac->prepare_request_memcpy(dmac, req);
}

static inline int
dma_start_request(struct dma_controller *dmac,
		  unsigned int channel)
{
	return dmac->start_request(dmac, channel);
}

static inline int
dma_stop_request(struct dma_controller *dmac,
                 unsigned int channel)
{
	return dmac->stop_request(dmac, channel);
}

static inline dma_addr_t
dma_get_current_pos(struct dma_controller *dmac,
                    unsigned int channel)
{
	return dmac->get_current_pos(dmac, channel);
}

extern int register_dma_controller(struct dma_controller *dmac);
extern struct dma_controller *find_dma_controller(int id);

#endif /* __ASM_AVR32_DMA_CONTROLLER_H */
